/**
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.webbeans.extension.scheduler;

import org.jboss.webbeans.scheduling.quartz.QuartzStarter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.enterprise.inject.spi.Bean;
import org.jboss.webbeans.BeanManagerImpl;
import org.jboss.webbeans.CurrentManager;
import org.jboss.webbeans.bean.RIBean;
import org.jboss.webbeans.bean.SimpleBean;
import org.jboss.webbeans.scheduling.util.WebBeansManagerUtils;
import org.jboss.webbeans.mock.MockDeployment;
import org.jboss.webbeans.mock.MockServletLifecycle;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

/**
 *
 * @author pete
 */
public abstract class AbstractCDITest
{

    MockServletLifecycle lifecycle;
    BeanManagerImpl manager;

    public AbstractCDITest()
    {
    }

    @AfterClass(alwaysRun = true)
    public void afterClass() throws Exception
    {
        lifecycle.endRequest();
        lifecycle.endSession();
        lifecycle.endApplication();
        lifecycle = null;
    }

    @BeforeClass
    public void beforeClass() throws Throwable
    {
        lifecycle = new MockServletLifecycle();
        lifecycle.initialize();
        MockDeployment deployment = lifecycle.getDeployment();
        final List<Class<? extends Object>> allBeansList = new ArrayList<Class<? extends Object>>(getDefaultWebBeans());
        allBeansList.addAll(getAdditionalWebBeans());
        deployment.getArchive().setBeanClasses(allBeansList);
        lifecycle.beginApplication();
        lifecycle.beginSession();
        lifecycle.beginRequest();
        manager = CurrentManager.rootManager();
    }

    @Test
    public void testDefaultBeans()
    {
        Map<Class<?>, Bean<?>> beans = new HashMap<Class<?>, Bean<?>>();
        for (Bean<?> bean : manager.getBeans()) {
            if (bean instanceof RIBean) {
                beans.put(((RIBean<?>) bean).getType(), bean);
            }
        }
        for (Class klass : getDefaultWebBeans()) {
            assert beans.containsKey(klass);
            assert beans.get(klass) instanceof SimpleBean;
        }
        WebBeansManagerUtils.getInstanceByType(manager, TestBean.class).ping();
    }

    public List<Class<? extends Object>> getDefaultWebBeans()
    {
        return Arrays.asList(TestBean.class, AnotherTestBean.class, QuartzStarter.class);
    }

    /**
     * Override in your tests to register specific beans with the manager.
     * @return
     */
    public List<Class<? extends Object>> getAdditionalWebBeans()
    {
        return Collections.EMPTY_LIST;
    }
}
